#ifndef __STARWIN_H
#define __STARWIN_H

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <termios.h>
#include <fcntl.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <vga.h>
#include <vgamouse.h>

typedef struct {
  int r;
  int g;
  int b;
  int a;
} swCOLOR;

extern int swWidth;
extern int swHeight;
extern int swScanWidth;
extern swCOLOR foreground;
extern swCOLOR background;

#define set_swcolor(c,R,G,B) { c.r=R; c.g=G; c.b=B; }
#define SETCOLOR(c) setcolor(c.r,c.g,c.b)

char * swFindVideoMode(char **modelist);
int swSetupVideo(char *mode);
int swClose(void);

extern int (*drawpixel)(int x,int y);
extern int (*getpixel)(int x,int y);
extern int (*drawline)(int x1,int y1,int x2,int y2);
extern int (*setcolor)(int r,int g,int b);
extern int (*bitblt)(int x,int y,int w,int h,char *bmp);
extern int (*getbitmap)(int x,int y,int w,int h,char *bmp);
extern int (*miximage)(unsigned char *dest,unsigned char *src,int w,int h);
extern int (*scroll)(int x,int y,int w,int h,int line);
extern int (*fillbox)(int x,int y,int w,int h);
extern int (*xorbox)(int x,int y,int w,int h);
extern int (*drawfont)(int x,int y,int w,int h,unsigned char *bmp);
extern int (*setforeground)(int r,int g,int b);
extern int (*setbackground)(int r,int g,int b);
extern unsigned char * (*create_image_xpm)(int *w,int *h,char **xpm);
extern unsigned char * (*create_buffer)(int w,int h);

extern int mouse_width;
extern int mouse_height;
extern int mouse_x;
extern int mouse_y;
extern int mice_x;
extern int mice_y;
extern int mice_button;
extern int mice_update;
extern int mouse_memsize;
extern int mouse_enable;
extern int mouse_painted;
extern unsigned char * mouse_bitmap;
extern unsigned char * mouse_screen;
extern unsigned char * mouse_paint;

typedef int (*MouseHandler)(int button,int x,int y,void *pocket);

extern int (*mice_grinit)(char **arrow_xpm);
int mice_init(char *dev,char **arrow_xpm);
int mice_recover(void);
int mice_move(int x,int y);
int mice_sethandie(MouseHandler handie,void *pocket);
int mice_default(int button,int x,int y,void *pocket);

/********************\
        Font
\********************/

typedef struct {
  unsigned int l;
  unsigned int h;
  unsigned int a;
} Byte2Info;

typedef struct {
  unsigned int    l;
  unsigned int    h;
  long            offset;
  char            bmfname[50];
  int             address[256];
  FILE *          fp;
} HBFRange;

typedef struct {
  int             mx;
  int             my;
  int             byte2range;
  int             totalrange;
  Byte2Info *     B2Info;
  int             segs;
  HBFRange *      SEG;
  int             bytes;
} HBFInfo;

int             hbfOpen(char *path,char *filename,HBFInfo *info);
unsigned char * hbfGetBitmap(HBFInfo *info,char *str,unsigned char *addr);
int             hbfClose(HBFInfo *info);
int             swCheckEncoding(char *str);

typedef struct {
  char * encoding;
  int width;
  int height;
  
  char * eng_fname;
  int eng_w;
  int eng_h;
  unsigned char * EFONT[256];
  
  char * dbcs_fname;
  char * dbcs_path;
  int dbcs_w;
  int dbcs_h;
  unsigned char * CBUFF;
  HBFInfo info;
} swFont;
extern swFont FONT;

int swLoadFont(void);
int swCloseFont(void);
int swputs(int x,int y,unsigned char *msg);
int swprintf(int x,int y,char *fmt,...);

/***************************\
    Star Window Functions
\***************************/
typedef struct {
  int x,y;
} swPoint;

typedef struct {
  int x1;
  int y1;
  int x2;
  int y2;
} swRect;

int swIsInside(int x,int y,swRect *RECT);
int swDrawBorder(swRect *RECT,swCOLOR *Lite,swCOLOR * Vampire);
int swDrawBox(swRect * RECT,swCOLOR * color,int up_down);

/*****************************\
            Button
\*****************************/
int swDrawButton(
      swRect * RECT,
      swCOLOR * color,
      swCOLOR * text,
      int up_down,
      char *fmt,...
    );

/*****************************\
             Menu
\*****************************/
typedef struct {
  char * menu;
  int id;
} swMenuItem;

typedef struct {
  int size;
  swMenuItem * menu;
} swMenu;

#define GRAVITY_NW 0
#define GRAVITY_NE 1
#define GRAVITY_SW 2
#define GRAVITY_SE 3

int swDoMenu(int x,int y,int gravity,swMenu *MENU);

/*****************************\
        Key Functions
\*****************************/
extern int swConsole;
int swSetupKey(char *dev_console);
int swCloseKey(void);
int swGetkey(void);
int swGetCH(void);
int swIsKeyAvailable(void);

/*****************************\
         Message Box
\*****************************/
int swMessageBox(char *title,char *btn,char *msg,...);
int swNoticeYesNo(char *title,char *yes,char *no,char *msg,...);

/******************************\
         TTY Functions
\******************************/
#define TTY_HISTORY_BUFF 512
typedef struct {
  swRect RECT;
  swCOLOR bkgnd;
  swCOLOR frgnd;
  swCOLOR ttyColor[16];
  int     fgidx;
  int     bgidx;
  int     reverse;
  int     intense;
  int     invert;

  int row;
  int col;
  int lines;
  int start;
  int idx,x;
  int end;
  unsigned char ** history;
  unsigned char ** color;
  
  int cur_x;
  int cur_y;
  int last_x;
  int last_y;
  int cursor_state;
  int dec_cursor_keys;
  int cursor_visible;
  
  int lf_mode;
  int insert_mode;
  
  int state;
  unsigned char BUF[256];
  int bufidx;
  int need_wrap;
  int autorepeat;
  int autowrap;
  int applic;
  int underline;
  int blink;
  int * tab_stops;
  
  unsigned char PARAM[256];
  int param;
  int question;
  
  int OriginMode;
  int Top;
  int Bottom;

  int pty;
} swTerminal;

int open_pty(char *ptyname);
pid_t open_tty(char *ptyname,int row,int col,int login);
swTerminal * swCreateTerminal(int x,int y,int row,int col);
int swDrawTerminal(swTerminal * TERM);
int swCloseTerminal(swTerminal * TERM);

#endif


